@@ -1,15 +1,20 @@
 
 from .r2convolution import R2Conv
 from .r2_transposed_convolution import R2ConvTransposed
+from .r2diffop import R2Diffop
 
 from .basisexpansion import BasisExpansion
 from .basisexpansion_blocks import BlocksBasisExpansion
+from .basisexpansion_singleblock import SingleBlockBasisExpansion, block_basisexpansion
 
 __all__ = [
     "R2Conv",
     "R2ConvTransposed",
+    "R2Diffop",
     # Basis Expansion
     "BasisExpansion",
     "BlocksBasisExpansion",
+    "SingleBlockBasisExpansion",
+    "block_basisexpansion",
 ]
 
