@@ -166,7 +166,8 @@ class R2ConvTransposed(EquivariantModule):
         # notice that `out_type` is used instead of `self.out_type` such that it works also when `groups > 1`
         if basisexpansion == 'blocks':
             self._basisexpansion = BlocksBasisExpansion(in_type, out_type,
-                                                        grid,
+                                                        basis_generator=self.space.build_kernel_basis,
+                                                        points=grid,
                                                         sigma=sigma,
                                                         rings=rings,
                                                         maximum_offset=maximum_offset,
